#!/usr/bin/env python
"""

Dump or list a range of shots from a database in a nice table

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: segysqldump.py,v 1.8 2005/09/27 17:36:52 schwehr Exp $

"""

# These values end up in the pydoc web page
__version__ = '$Revision: 1.8 $' # See man ident
__date__ = '$Date: 2005/09/27 17:36:52 $' # FIX: pull out just the date
__author__ = 'Kurt Schwehr'
__credits__ = """LSD inspired... the program, not the drug"""

import sqlite
import os
import sys
import math

BOMBASTIC= 4
VERBOSE  = 3
TRACE    = 2
TERSE    = 1
ALWAYS   = 0


from optparse import OptionParser

vcvs = "$Revision: 1.8 $"
vcvs = vcvs[vcvs.find(':')+2:]
vcvs = vcvs[:vcvs.find('$')-1]
version_cvs = vcvs
del (vcvs) # stop polution

# Track the version of segy-py.  If I use setup.py, which one gets imported?
from segy_version import VERSION

myparser = OptionParser(usage="%prog [options] key1 key2 key3 ...",
                      version="%prog "+VERSION+' - '+version_cvs)
myparser.add_option('-d','--database',dest='dbfilename',default='segy.db',
                    help='Database containing imported segy file and trace header info [default: %default]')

myparser.add_option('-n','--nth-trace',dest='nthTrace',type='int',default='500',
                    help='How many trace entries to skip before writing out a label [default: %default]')


myparser.add_option('-w','--width',type="int",dest='width',default=12,
                    help='Character width of each field [default: %default]')


myparser.add_option('-v',action="count",dest='verbosity',default=0,
                    help='how much information to give.  Specify multiple times to increase verbosity')
myparser.add_option('--verbosity',dest='verbosity',type='int',
                    help='Specify verbosity.  Should be in the range of '
                    +str(ALWAYS)+'...'+str(BOMBASTIC)+' (None...Bombastic)')

myparser.add_option('-r','--raw', action="store_true", dest='rawCoords',default=False,
                    help='Do not translate X and Y.  Show them in the orignal segy form.')

myparser.add_option('--min-sec',action='store_true',dest='minSec',default=False,
                    help='Output coordinates in degrees min seconds format')

myparser.add_option('-u','--from-utm',action="store_true",dest='fromUTM',default=False,
                    help='Assume x and y are in UTM and convert to Long/Lat')
myparser.add_option('-z','--utm-zone',type="int",dest='utmZone',default=None,
                    help='Specify the UTM zone if converting from UTM to Long/Lat')
myparser.add_option('-c','--coord-unit',type="int",dest='forceCoordUnit',default=-1,
                    help='Force a SEGY coordinate system, e.g. 2 == seconds of arc.  Use this for short format databases. [default: follow file]')

myparser.add_option('-x','--flip-x',dest='flipX',default=False,action='store_true',
                    help='Multiply the x coordinate by -1')
myparser.add_option('-y','--flip-y',dest='flipY',default=False,action='store_true',
                    help='Multiply the x coordinate by -1')
myparser.add_option('-H','--force-hemisphere',type="int",dest='forceHemisphere',default=-1,
                    help='Force hemisphere for shots that have bad navigation.'+
                    '  0-NW,  1-NE,  2-SW,  3-SE')

myparser.add_option('-l','--list-keys',dest='listKeys',default=False,action='store_true',
                    help='List the possible keys and exit')

myparser.add_option('-L','--limit',dest='limit',default=None,type='int',
                    help='Limit the output to this many lines [default: %default]')



from segysqlgmt import CoordTransform

######################################################################

def hasField(db_cx, fieldName, table):
    """Return true if a field exists in a table
    FIX: the dependency on the spaces before the fieldName is dangerous
    """

    cu = db_cx.cursor()
    cu.execute('select sql from sqlite_master where name = "'+table+'"')
    row = cu.fetchone()
    if -1 == row['sql'].find(' '+fieldName+' '): return False
    return True

######################################################################
if __name__ == "__main__":
    (options,args) = myparser.parse_args()

    if options.verbosity >= TRACE:
        print "Verbosity = ", options.verbosity

    cx = sqlite.connect(options.dbfilename)

    if options.listKeys:
        cu = cx.cursor()
        #cu.execute('select sql from sqlite_master where name = "segyTrace"')
        #row = cu.fetchone()
        #table=row['sql']
        #print table
        #dummy,table = table.split('(')
        #print table
        cu.execute('select * from segyTrace limit 1')
        row = cu.fetchone()
        count = 0
        for key in row.keys():
            print key,'\t',
            count += 1
            if count > 6:
                print
                count = 0
        print
        sys.exit('List completed.  Exiting.')

    ct = None
    if not options.rawCoords:
        if options.verbosity>=VERBOSE: print "setting up coord transform"
        if options.fromUTM and options.verbosity >= TRACE:
            print "Setting up UTM to LonLat conversions for zone: ", options.utmZone
        ct = CoordTransform(cx,options)
        
    cu = cx.cursor()
       

    fields = []
    fields.append('fileKey')
    fields.append('traceNumber')
    fields.append('X')
    fields.append('Y')
    #fields.append('GrpX')
    #fields.append('GrpY')
    fields.append('EnsembleNo')
    fields.append('FieldRecNo')
    fields.append('Xensemble')
    fields.append('Yensemble')
    #fields.append('Year')
    #fields.append('Day')
    #fields.append('Hour')
    #fields.append('Min')
    fields.append('Shotpoint')
    fields.append('TraceNoEnsemble')

    # All extra args are fields to print
    for field in args:
        fields.append(field)

    fieldsShort = []
    for field in fields:
        if hasField(cx, field, 'segyTrace'):
            fieldsShort.append(field)
            if options.verbosity>=BOMBASTIC: print 'yes, has '+field
        else:
            if options.verbosity>=TRACE: print ' NO! Missing '+field

    print fields
    print fieldsShort
    
    fieldStr = fieldsShort[0]
    for i in fieldsShort[1:]:
        fieldStr += ','+i

    fields = fieldsShort
    
    searchStr  = "SELECT "+fieldStr
    searchStr += " FROM segyTrace"
    searchStr += " WHERE traceNumber % "+str(options.nthTrace)+"= 0"
    #if (None != options.minFileKey): searchStr += ' fileKey>='+str(options.minFileKey)
    #if (None != options.minFileKey) and (None != options.maxFileKey): searchStr += ' and'
    #if (None != options.maxFileKey): searchStr += ' fileKey<='+str(options.maxFileKey)

    if options.limit != None: searchStr += ' limit '+str(options.limit)+' '
    searchStr += ';'

    if options.verbosity>=BOMBASTIC: print searchStr

    width = options.width
    cu.execute(searchStr)
    count = 0;
    for row in cu.fetchall():
        if count%50 == 0:
            print
            for field in fields:
                while len(field)<width: field += ' '
                while len(field)>width: field = field[:-1]
                print field,' ',
            print '\n'
        count += 1
        x = row['X']
        y = row['Y']
        if options.verbosity>=BOMBASTIC: print 'raw xy: ',x,y
        #if options.fromUTM:
        if not options.rawCoords:
            x_,y_ = ct.getLonLat(x,y)
            if options.verbosity>=VERBOSE: print "converted",x,y,' to ',x_,y_
            x,y = x_,y_
        else:
            if options.verbosity>=VERBOSE: print "have some raw"

        for field in fields:

            val = str(row[field])
            #if options.fromUTM:
            if not options.rawCoords:
                if ('X'==field or 'Y'==field) and options.minSec:
                    # Degrees minutes seconds
                    start = None
                    if 'X' == field: start = x
                    if 'Y' == field: start = y
                    deg = int(math.floor(math.fabs(start)))
                    #if start < 0: deg += 1
                    minutes = int(math.floor((math.fabs(start) % 1) * 60))
                    seconds = ((math.fabs(start) %1 ) - minutes/60.)*3600
                    val = str(deg)+' '+str(minutes)+' '+("%02.2f" %seconds)+' '
                    if 'X' == field:
                        if start < 0: val += 'W'
                        else: val += 'E'
                    if 'Y' == field:
                        if start < 0: val += 'S'
                        else: val += 'N'
                    
                else:
                    if 'X' == field: val = str(x)
                    if 'Y' == field: val = str(y)

            while len(val)<width: val += ' '
            if len(val)>width: val = val[:width]
            #if len(val)>width: val = val[2:]
            print val,' ',

        print
