#!/usr/bin/env python

"""
Find water bottom using any number of algorithms.  Add your own!!

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: segywaterbottom.py,v 1.2 2005/09/26 05:59:44 schwehr Exp $

"""

# These values end up in the pydoc web page
__version__ = '$Revision: 1.2 $' # See man ident
__date__ = '$Date: 2005/09/26 05:59:44 $' # FIX: pull out just the date
__author__ = 'Kurt Schwehr'

import os
import sys

BOMBASTIC= 4
VERBOSE  = 3
TRACE    = 2
TERSE    = 1
ALWAYS   = 0


from segy import Segy
import segy

from optparse import OptionParser

# What version do we want to track?  This would be for cvs of this file
vcvs = "$Revision: 1.2 $"
vcvs = vcvs[vcvs.find(':')+2:]
vcvs = vcvs[:vcvs.find('$')-1]
version_cvs = vcvs
del (vcvs) # stop polution

# Track the version of segy-py.  If I use setup.py, which one gets imported?
from segy_version import VERSION

myparser = OptionParser(usage="%prog [options] segyfile1 segyfile2 ... segyfileN",
                      version="%prog "+VERSION+' - '+version_cvs)

myparser.add_option('-d','--driver',dest="drv",
                    help="specify which driver to parse data file.  [default: %default]",
                    default="segy_drv_rev1")
myparser.add_option('-f','--file',dest="file",action="append", type="string",
                    help="specify segy file to examine.  Specify one or more of these")

myparser.add_option('-v','--verbose',action="count",dest='verbosity',default=0,
                    help='how much information to give.  Specify multiple times to increase verbosity')
myparser.add_option('--verbosity',dest='verbosity',type='int',
                    help='Specify verbosity.  Should be in the range of '
                    +str(ALWAYS)+'...'+str(BOMBASTIC)+' (None...Bombastic)')
myparser.add_option('--noisy',dest='verbosity', action='store_const', const=2*BOMBASTIC,
                    help='Go for the max verbosity')

myparser.add_option('-t','--threshold',dest='threshold',action='append',type='float',
                    help='Trigger when a threshold level is reached.  Okay to specify multiple thresholds')

myparser.add_option('-s','--skip',dest='skip',type='int',
                    help='How many traces to skip between those that we check', default=0)

myparser.add_option('-T','--time',dest='time',action='store_true', default=False,
                    help='Return matches as time in seconds rather than trace data offset')

myparser.add_option('-k','--key',dest='key',type='string', default=None,
                    help='Reference key to put in the 2nd column')

# FIX: what is the best output format?
# FIX: make each algorithm be a subclass of a parent class or something.
# What kind of API?  Want access to neighbors.  Generic or depend on segy class?

def offset2seconds(segy, trace, offset):
    """Convert the data array offset in the trace data block to a two way travel time

    segy - Segy data file object
    trace - trace number starting from 1
    offset - what bin the sample is in
    """
    #import pdb
    #pdb.set_trace()
    
    #print 
    traceHeader = segy.getTraceHdr(trace)

    #from IPython.Shell import IPShellEmbed
    #ipshell = IPShellEmbed(argv=[])
    #ipshell() 

    interval = traceHeader.getHeader('SampleInterval')/1000000.
    delay    = traceHeader.getHeader('Delay')/1000.
    try:
        time = delay + interval * offset
    except TypeError:
        print 'ERROR in offset2seconds:', trace, offset, type(delay),type(interval), type(delay)
        print '  ... ', delay, interval, offset
        time = delay
    return time



if __name__ == '__main__':
    (options,args) = myparser.parse_args()
    v = options.verbosity
    if v >= VERBOSE:   print '# Option values: ', options
    if v >= BOMBASTIC: print '# Arg    values: ', args

    for file in options.file:
        print
        print '# FILE: ',file
        s = Segy(file,drivername=options.drv,verbosity=options.verbosity)
        #minSample =  10e99
        #maxSample = -10e99
        for traceNumber in range(1,s.getNumberOfTraces()+1):
            if (traceNumber%options.skip != 0): continue
            data = s.getTraceData(traceNumber)
            #minLocal = min(data)
            #maxLocal = max(data)
            #if (minLocal<minSample): minSample = minLocal
            #if (maxLocal>maxSample): maxSample = maxLocal
            #print i,minLocal,maxLocal,minSample,maxSample
            thresholdPositions=[]
            for thresh in options.threshold:
                result = None
                for position in range(len(data)):
                    #print position, data[position], thresh
                    if data[position]>thresh:
                        result=position
                        break
                thresholdPositions.append(result)
            print traceNumber,
            if options.key != None:
                keyval = s.getTraceHdr(traceNumber).getHeader(options.key)
                print keyval,
            if options.time:
                for p in thresholdPositions:
                    time = None
                    if None != p: time = offset2seconds(s,traceNumber,p)
                    print time ,' ',
            else:
                for p in thresholdPositions: print p,' ',
            print

                
                        

